import frappe
import json

@frappe.whitelist()
def change_contract_status(doc, status):
    """
    Validates if property already on rent between the from and to dates
    """
    doc = json.loads(doc)

    if status == "Hold":
        doc = frappe.get_doc("Property Contract", doc["name"])
        doc.status = status
        doc.save()

        unit_doc = frappe.get_doc("Unit", doc.unit)
        unit_doc.status = "Vacant"
        unit_doc.save()
        return

    # Get Property Contaract if from date is between new from and new to date
    filters = {
        "rent_start_date": ["between", [doc["rent_start_date"], doc["rent_end_date"]]],
        "docstatus": 1,
        "unit_name": doc["unit_name"],
        "status": ["!=", "Hold"]
    }
    
    property_contract = frappe.get_list("Property Contract",
    fields = ["name"],
    filters = filters)

    if len(property_contract)> 0:
        frappe.throw(f"This property unit already contain a contract <a href='#Form/Property%20Contract/{property_contract[0].name}'>{property_contract[0].name}</a> that not ended yet.")


    # Get Property Contaract if to date is between new from and new to date
    filters = {
        "rent_end_date": ["between", [doc["rent_start_date"], doc["rent_end_date"]]],
        "docstatus": 1,
        "unit_name": doc["unit_name"],
        "status": ["!=", "Hold"]
    }
    
    property_contract = frappe.get_list("Property Contract",
    fields = ["name"],
    filters = filters)

    if len(property_contract)> 0:
        frappe.throw(f"This property unit already contain a contract <a href='#Form/Property%20Contract/{property_contract[0].name}'>{property_contract[0].name}</a> that not ended yet.")


    # Get Property Contaract if from date is after new from date and to date is before new to date
    filters = {
        "rent_start_date": ["<=", doc["rent_start_date"]],
        "rent_end_date": [">=", doc["rent_end_date"]],
        "docstatus": 1,
        "unit_name": doc["unit_name"],
        "status": ["!=", "Hold"]
    }
    
    property_contract = frappe.get_list("Property Contract",
    fields = ["name"],
    filters = filters)

    if len(property_contract)> 0:
        frappe.throw(f"This property unit already contain a contract <a href='#Form/Property%20Contract/{property_contract[0].name}'>{property_contract[0].name}</a> that not ended yet.")


    doc = frappe.get_doc("Property Contract", doc["name"])
    doc.status = status
    doc.save()

    unit_doc = frappe.get_doc("Unit", doc.unit_name)
    unit_doc.status = "Leased"
    unit_doc.save()
