# -*- coding: utf-8 -*-
# Copyright (c) 2021, Havenir Solutions and contributors
# For license information, please see license.txt

from __future__ import unicode_literals
import frappe
from frappe.model.document import Document
from frappe.utils import flt


class DiscountRequest(Document):
    def on_submit(self):
        self.add_remove_discount(type='Add')

    def on_cancel(self):
        self.add_remove_discount(type='Remove')

    def get_brand_names(self):
        list_items = []
        brands = frappe.get_all("Tenant Brand Name", filters={"parent": self.customer}, fields=['tenant_brand_name'],
                                order_by="tenant_brand_name")
        if brands:
            for item in brands:
                list_items.append(item["tenant_brand_name"])

        return list_items

    def add_remove_discount(self, type):
        filters = {"parent": self.property_contract, "date": ["between", [self.start_date, self.end_date]],
                   "invoice": ["=", '']}
        if self.apply_discount_on != "All":
            filters['reason'] = self.apply_discount_on
        payments = frappe.db.get_all("Contract Payments", filters=filters, order_by="date")
        for row in payments:
            doc = frappe.get_doc("Contract Payments", row["name"])
            if type == 'Add':
                doc.amount -= (
                    ((flt(self.percentage) / 100) * flt(doc.amount)) if self.discount_type == 'Percentage' else flt(
                        self.amount))
                doc.remarks = ' تم تطبيق خصم على هذا الشهريقيمة <br>' + str((((flt(self.percentage) / 100) * flt(
                    doc.amount)) if self.discount_type == 'Percentage' else flt(self.amount))) + "<br>" + str(self.name)
                doc.discount_applied = 1
                doc.discount_refrence = self.name
            else:
                doc.amount = (( (flt(doc.amount) * 100) / flt(100 - flt(self.percentage)) ) if self.discount_type == 'Percentage' else (flt(self.amount)+flt(doc.amount)))
                doc.remarks = ''
                doc.discount_applied = 0
                doc.discount_refrence = ''

            doc.flags.ignore_validate_update_after_submit = True
            doc.save()
            frappe.db.commit()
