# -*- coding: utf-8 -*-
# Copyright (c) 2021, Havenir Solutions and contributors
# For license information, please see license.txt

from __future__ import unicode_literals
import frappe
from frappe import _
import datetime
# from datetime import datetime, timedelta
import dateutil
import calendar
from calendar import weekday, monthrange
from collections import Counter
from dateutil.relativedelta import relativedelta
from dateutil.rrule import rrule, MONTHLY, YEARLY
from frappe.model.document import Document
from frappe.utils import flt, date_diff, getdate, cint, nowdate,get_datetime


class PropertyContract(Document):
    def validate(self):
        if self.is_new():
            self.add_payment()
        # self.add_contract_annual_raise()

    def before_submit(self):
        """
        Validates if property already on rent between the from and to dates
        """

        # Get Property Contaract if from date is between new from and new to date
        filters = {
            "rent_start_date": ["between", [self.rent_start_date, self.rent_end_date]],
            "docstatus": 1,
            "unit": self.unit,
            "status": ["!=", "Hold"]
        }
        
        property_contract = frappe.get_list("Property Contract",
        fields = ["name"],
        filters = filters)

        if len(property_contract)> 0:
            frappe.throw(f"This property unit already contain a contract <a href='#Form/Property%20Contract/{property_contract[0].name}'>{property_contract[0].name}</a> that not ended yet.")


        # Get Property Contaract if to date is between new from and new to date
        filters = {
            "rent_end_date": ["between", [self.rent_start_date, self.rent_end_date]],
            "docstatus": 1,
            "unit": self.unit,
            "status": ["!=", "Hold"]
        }
        
        property_contract = frappe.get_list("Property Contract",
        fields = ["name"],
        filters = filters)

        if len(property_contract)> 0:
            frappe.throw(f"This property unit already contain a contract <a href='#Form/Property%20Contract/{property_contract[0].name}'>{property_contract[0].name}</a> that not ended yet.")


        # Get Property Contaract if from date is after new from date and to date is before new to date
        filters = {
            "rent_start_date": ["<=", self.rent_start_date],
            "rent_end_date": [">=", self.rent_end_date],
            "docstatus": 1,
            "unit": self.unit,
            "status": ["!=", "Hold"]
        }
        
        property_contract = frappe.get_list("Property Contract",
        fields = ["name"],
        filters = filters)

        if len(property_contract)> 0:
            frappe.throw(f"This property unit already contain a contract <a href='#Form/Property%20Contract/{property_contract[0].name}'>{property_contract[0].name}</a> that not ended yet.")

        diff_days = date_diff(self.rent_end_date,getdate())
        if flt(diff_days) > 0:
            frappe.db.set_value("Unit", self.unit, "status", "Leased")

    def add_contract_annual_raise(self):
        self.contract_annual_raise = []
        start_date = datetime.datetime.strptime(self.rent_start_date, "%Y-%m-%d")
        duration = self.duration - 1 if self.duration > 1 else self.duration
        month = dateutil.relativedelta.relativedelta(months=duration)
        end_date = start_date + month

        for row in rrule(YEARLY, dtstart=start_date, until=end_date):
            self.append('contract_annual_raise', {
                'date': row.date(),
                'amount': 0,
            })

    def add_payment(self):

        self.payments = []
        start_date = datetime.datetime.strptime(self.rent_start_date, "%Y-%m-%d")
        duration = self.duration - 1 if self.duration > 1 else self.duration
        month = dateutil.relativedelta.relativedelta(months=duration)
        end_date = start_date + month
        
        if not self.annual_raise_start_date:
            duration_to_add = 13 - start_date.month
            months = dateutil.relativedelta.relativedelta(months = duration_to_add)
            self.annual_raise_start_date = start_date + months
            self.annual_raise_start_date = self.annual_raise_start_date.strftime('%Y-%m-%d')

        annual_raise_start_date = datetime.datetime.strptime(str(self.annual_raise_start_date), "%Y-%m-%d")
        service_charge_start_date = datetime.datetime.strptime(str(self.service_charge_start_date), "%Y-%m-%d")
        
        frequency = 0
        if self.type == 'Customized c':
            if self.payments_scheduling:
                frequency = self.payments_scheduling
        else:
            if self.type == 'Monthly':
                frequency = 1
            if self.type == 'Quarterly':
                frequency = 4
            if self.type == 'Half Yearly':
                frequency = 6
            if self.type == 'Annually':
                frequency = 12

        annual_raise_amount = self.annual_raise_amount
        rent = (self.total_rent_per_month if not self.split_indoor_from_outdoor_rent else self.indoor_rent_month)
        year = annual_raise_start_date.year
        service_year = start_date.year
        service_charge =0.0

        if self.split_indoor_from_outdoor_service:
            service_charge = self.service_charge_per_month or 0
        else:
            service_charge = self.total_services_rent or 0

        flag=1

        service_charge_frequency = 1
        if self.service_charges_type == 'Annually':
            service_charge_frequency = 12
        if flt(service_charge) > 0 :
            for row in rrule(MONTHLY, interval=service_charge_frequency, dtstart=start_date, until=end_date):
                if (self.service_raise_type == "Annual Regular"):
                    if row.month == service_charge_start_date.month and row.year >= service_charge_start_date.year:
                    # if row.year >service_year:
                        service_year=row.year
                        if self.annual_service_charges == "Percentage":
                            service_charge += service_charge / 100 * int((self.amount if self.amount else "0"))
                        else:
                            service_charge += int(self.amount)

                if (self.service_raise_type == "Annual Irregular"):
                        ourdoor_service_dict = {}
                        for sub_row in self.service_raise_details:
                            date = datetime.datetime.strptime(sub_row.date, "%Y-%m-%d")
                            date = date.strftime("%d-%m-%Y")
                            ourdoor_service_dict[date] = f'{sub_row.annual_raise_type}_{sub_row.annual_raise_amount}'

                        month = '{:02d}'.format(row.month)
                        day = '{:02d}'.format(row.day)

                        fieldname = f'{day}-{month}-{row.year}'

                        if fieldname in ourdoor_service_dict and ourdoor_service_dict[fieldname]:
                            splitted_field = ourdoor_service_dict[fieldname].split("_")
                            annual_raise_type = splitted_field[0]
                            annual_raise_amount = splitted_field[1]
                            # print("===================>" + str(annual_raise_amount))
                            if annual_raise_type == "Percentage":
                                service_charge += service_charge / 100 * int(annual_raise_amount)
                            else:
                                service_charge += int(annual_raise_amount)

                self.append('payments', {
                'date': row.date(),
                'amount': service_charge or 0,
                'reason':'Services Charges'+("" if not self.split_indoor_from_outdoor_service else " - InDoor")
                })

        if self.split_indoor_from_outdoor_service:
            service_charge = self.outdoor_service_charge_per_month
            year = service_charge_start_date.year
            service_year = start_date.year
            if service_charge > 0:
                for row in rrule(MONTHLY, interval=service_charge_frequency, dtstart=start_date, until=end_date):
                    if (self.service_raise_type == "Annual Regular"):
                        if row.month == service_charge_start_date.month and row.year >= service_charge_start_date.year:
                        # if row.year > service_year:
                            service_year = row.year
                            if self.annual_service_charges == "Percentage":
                                service_charge += service_charge / 100 * int(self.amount)
                            else:
                                service_charge += int(self.amount)

                    if (self.service_raise_type == "Annual Irregular"):
                        ourdoor_service_dict = {}
                        for sub_row in self.service_raise_details:
                            date = datetime.datetime.strptime(sub_row.date, "%Y-%m-%d")
                            date = date.strftime("%d-%m-%Y")
                            ourdoor_service_dict[date] = f'{sub_row.annual_raise_type}_{sub_row.annual_raise_amount}'

                        month = '{:02d}'.format(row.month)
                        day = '{:02d}'.format(row.day)

                        fieldname = f'{day}-{month}-{row.year}'

                        if fieldname in ourdoor_service_dict and ourdoor_service_dict[fieldname]:
                            splitted_field = ourdoor_service_dict[fieldname].split("_")
                            annual_raise_type = splitted_field[0]
                            annual_raise_amount = splitted_field[1]
                            # print("===================>" + str(annual_raise_amount))
                            if annual_raise_type == "Percentage":
                                service_charge += service_charge / 100 * int(annual_raise_amount)
                            else:
                                service_charge += int(annual_raise_amount)

                    self.append('payments', {
                        'date': row.date(),
                        'amount': service_charge or 0,
                        'reason': 'Services Charges'+" - " +"OutDoor"
                    })

        for row in rrule(MONTHLY, interval=frequency, dtstart=start_date, until=end_date):
            if(self.annual_raise == "Annual Regular"):
                if row.year >= annual_raise_start_date.year:
                    #if row.month >= annual_raise_start_date.month or row.year > annual_raise_start_date.year:
                    if row.month == annual_raise_start_date.month and row.year >= annual_raise_start_date.year:
                        if row.year > year :
                            year=row.year
                            flag=1

                        if flag == 1:
                            if self.annual_raise_type == "Percentage":
                                rent += rent / 100 * int(annual_raise_amount)
                            else:
                                rent += int(annual_raise_amount)
                            flag = 0

            if(self.annual_raise == "Annual Irregular"):
                contract_dict = {}
                for sub_row in self.contract_annual_raise:
                    date = datetime.datetime.strptime(sub_row.date, "%Y-%m-%d")
                    date = date.strftime("%d-%m-%Y")
                    contract_dict[date] = f'{sub_row.annual_raise_type}_{sub_row.annual_raise_amount}'


                month = '{:02d}'.format(row.month)
                day = '{:02d}'.format(row.day)

                fieldname = f'{day}-{month}-{row.year}'

                if fieldname in contract_dict and contract_dict[fieldname]:
                    splitted_field = contract_dict[fieldname].split("_")
                    annual_raise_type = splitted_field[0]
                    annual_raise_amount = splitted_field[1]
                    # print("===================>" + str(annual_raise_amount))
                    if annual_raise_type == "Percentage":
                        rent += rent / 100 * int(annual_raise_amount)
                    else:
                        rent += int(annual_raise_amount)

            self.append('payments', {
                'date': row.date(),
                'amount': rent or 0,
                'reason':'Rent'+(' - indoor' if self.split_indoor_from_outdoor_rent else '')
            })

        if self.split_indoor_from_outdoor_rent:
            flag = 1
            rent = self.outdoor_rent_month
            if rent > 0:
                for row in rrule(MONTHLY, interval=frequency, dtstart=start_date, until=end_date):
                    if (self.annual_raise == "Annual Regular"):
                        if row.year >= annual_raise_start_date.year:
                            #if row.month >= annual_raise_start_date.month or row.year > annual_raise_start_date.year:
                            if row.month == annual_raise_start_date.month and row.year >= annual_raise_start_date.year:
                                if row.year > year:
                                    year = row.year
                                    flag = 1

                                if flag == 1:
                                    if self.annual_raise_type == "Percentage":
                                        rent += rent / 100 * int(annual_raise_amount)
                                    else:
                                        rent += int(annual_raise_amount)
                                    flag = 0

                    if (self.annual_raise == "Annual Irregular"):
                        contract_dict = {}
                        for sub_row in self.contract_annual_raise:
                            date = datetime.datetime.strptime(sub_row.date, "%Y-%m-%d")
                            date = date.strftime("%d-%m-%Y")
                            contract_dict[date] = f'{sub_row.annual_raise_type}_{sub_row.annual_raise_amount}'

                        month = '{:02d}'.format(row.month)
                        day = '{:02d}'.format(row.day)

                        fieldname = f'{day}-{month}-{row.year}'

                        if fieldname in contract_dict and contract_dict[fieldname]:
                            splitted_field = contract_dict[fieldname].split("_")
                            annual_raise_type = splitted_field[0]
                            annual_raise_amount = splitted_field[1]

                            if annual_raise_type == "Percentage":
                                rent += rent / 100 * int(annual_raise_amount)
                            else:
                                rent += int(annual_raise_amount)

                    self.append('payments', {
                        'date': row.date(),
                        'amount': rent or 0,
                        'reason': 'Rent' + ' - outdoor'
                    })

        if not self.is_new():
            self.save()

    def on_cancel(self):
        frappe.db.set_value("Unit", self.unit, "status", "Vacant")

    def create_row_invoice(self,rowid):
        create_sales_invoice(self.name,rowid)

    def get_brand_names(self):
        list_items = []
        brands = frappe.get_all("Tenant Brand Name", filters={"parent": self.renter}, fields=['tenant_brand_name'], order_by="tenant_brand_name")
        if brands:
            for item in brands:
                list_items.append(item["tenant_brand_name"])

        return list_items

    def update_duration(self):
        duration = self.get_dates_diff(self.rent_start_date,self.rent_end_date)
        # frappe.msgprint(str(duration))

        duration_months = (cint(duration["years"]) * 12) + cint(duration["months"]) + (1 if cint(duration["days"]) > 0 or getdate(self.rent_end_date).day == 1 else 0)
        # frappe.msgprint(str(duration_months))
        self.duration = duration_months

        return duration_months

    def get_dates_diff(self,a, b):
        a = getdate(a)
        b = getdate(b)
        diff_dict = {}
        # a = datetime(a.year, a.month, a.day)
        # b = datetime(b.year, b.month, b.day)
        delta = dateutil.relativedelta.relativedelta(b, a)
        diff_dict["years"] = delta.years
        diff_dict["months"] = delta.months
        diff_dict["days"] = delta.days

        return diff_dict

def create_sales_invoice(docname="",rowid=""):
    print("Starting Creation..........")
    condiction = ""
    if docname and rowid:
        row_doc = frappe.get_doc("Contract Payments",rowid)
        condiction = " and C.parent = '%s' " % docname
        # condiction += " and C.`name` = '%s' " % rowid
        condiction += " and C.date = '%s' " % row_doc.date
    invoices = frappe.db.sql("""
        select *,P.`name` contract_name,C.date invoice_date,C.amount row_amount
            from `tabProperty Contract` P
            inner JOIN
          `tabContract Payments` C 
            on P.`name` = C.parent
            -- and CURRENT_DATE() >= C.date
            and P.rent_item is not NULL
            and P.service_item is not NULL
            -- and P.docstatus = 1
            and C.invoice is Null
            and C.amount > 0
            {0}
            order by date
        """.format(condiction),as_dict=True,debug=True)

    if invoices:
        # for invoice in invoices:
        #     s_name = create_sales_invoices(invoice)
        #     print(str(s_name.name)+"  Invoice Created.")
        s_name = create_sales_invoices(invoices)
    print("Finished.")

def create_sales_invoices(invoices):
    # frappe.throw(str(args))
    default_company = frappe.db.get_default("Company")
    si = frappe.new_doc("Sales Invoice")

    if isinstance(invoices,list):
        args = frappe._dict(invoices[0])
    else:
        args = frappe._dict(invoices)

    invoice_date  = args.date or nowdate()
    si.posting_date = invoice_date

    debit_to = frappe.db.get_value("Company", default_company, 'default_receivable_account')
    if debit_to:
        si.debit_to = debit_to  # Default Receivable Account
    else:
        frappe.throw(_("Please set Default Receivable Account in Company " + default_company))

    si.set_posting_time = 1
    si.postint_date = args.invoice_date
    si.due_date = args.invoice_date

    si.customer = args.renter
    si.tax_id = frappe.db.get_value("Customer",args.renter,"tax_id")
    si.debit_to = debit_to

    print(str(si.postint_date)+"================================================="+str(si.due_date))

    #Accounting Dimensions
    si.property = args.property_name
    si.unit = args.unit
    si.property_contract = args.contract_name
    invoice_curerncy = frappe.db.get_value('Customer',  args.renter,  "default_currency") or frappe.get_cached_value('Company',  default_company,  "default_currency")

    si.currency=invoice_curerncy
    conversion_rate = 1
    if invoice_curerncy !="EGP":
        conversion_rate = get_conversion_rate(invoice_curerncy,invoice_date)
        if conversion_rate == 0:
            message = _("Please Insert Currency Exchange for day {0}".format(invoice_date))
            frappe.throw(frappe.bold(message))

    si.conversion_rate = conversion_rate
    tax_list = []
    old_template = ""
    for item in invoices:
        default_income_account = None
        item_income_account = frappe.db.get_value("Item Default", {"parent":(item.rent_item if item.reason =="Rent" else item.service_item)}, 'income_account')
        company_default_income_account = frappe.db.get_value("Company", default_company, 'default_income_account')

        if item_income_account:
            default_income_account = item_income_account
        elif company_default_income_account:
            default_income_account = company_default_income_account
        else:
            frappe.throw(_("Please set Default Income Account in Company " + default_company))

        cost_center = frappe.db.get_value("Company", default_company, 'cost_center')
        if not cost_center:
            frappe.throw(_("Please set Default Cost Center in Company " + default_company))


        tax_template_name = ""
        if item.reason =="Services Charges" and item.service_sales_taxes_and_charges_template:
            tax_template_name = item.service_sales_taxes_and_charges_template
        elif item.reason =="Rent" and item.rent_sales_taxes_and_charges_template:
            tax_template_name = item.rent_sales_taxes_and_charges_template

        si.append("items", {
            "item_code":  (item.rent_item if item.reason =="Rent" else item.service_item) ,
            "item_name": (item.rent_item_name if item.reason =="Rent" else item.service_item),
            "qty": 1,
            "item_tax_template": (tax_template_name if tax_template_name else "") ,
            "rate": item.row_amount ,
            "uom": item.uom or "Unit",
            "stock_uom": item.uom or "Unit",
            "income_account": default_income_account,# Default Income Account
            "cost_center": cost_center ,
            "conversion_factor": 1,
            "description" : item.reason+" for contract "+  str(item.contract_name)
        })


        if tax_template_name:
            if old_template=="":
                old_template = tax_template_name

            tax_data = frappe.db.sql("""
                select *
                    from	`tabItem Tax Template Detail`
                        where parent='{tax_template_name}'
                    
            """.format(tax_template_name=tax_template_name),as_dict=True,debug=False)

            if tax_data:
                tax_data = tax_data[0]
                if old_template == tax_template_name:
                    tax_list = []

                tax_list.append({
                    "charge_type":"On Net Total",
                    "account_head":tax_data["tax_type"],
                    "description":tax_data["parent"],
                    "rate": 0 , #tax_data["tax_rate"],
                    "tax_amount":flt(tax_data["tax_rate"]) * flt(item.row_amount),
                    "total":(flt(tax_data["tax_rate"]) * flt(item.row_amount)) + flt(item.row_amount),
                    "base_tax_amount":flt(tax_data["tax_rate"]) * flt(item.row_amount),
                    "base_total":(flt(tax_data["tax_rate"]) * flt(item.row_amount)) + flt(item.row_amount)
                })

    for item in tax_list:
        si.append("taxes",item)

    Sinv_ID = None
    try:
        Sinv_ID = si.insert(ignore_mandatory = True)
        for row in invoices:
            frappe.db.sql("""update `tabContract Payments` set invoice = '{0}' where `name` = '{1}' """.format(Sinv_ID.name,row.name),debug=True)

        return Sinv_ID
    except Exception as e:
        print("Error......"+str(e))
        frappe.throw(_(e))

def get_conversion_rate(invoice_curerncy,invoice_date):
    conversion_rate = 0
    exchange_rate =  frappe.db.sql("""
        select exchange_rate 
            from `tabCurrency Exchange`	 
                where from_currency ='{invoice_curerncy}' 
                    and to_currency='EGP'	 
                    and date ='{invoice_date}'
    """.format(invoice_curerncy=invoice_curerncy,invoice_date=invoice_date),as_dict=True)
    if exchange_rate:
        conversion_rate =  exchange_rate[0]["exchange_rate"]

    return conversion_rate



def update_unit_status():
    units = frappe.db.get_list("Property Contract",filters={"docstatus":1,"rent_end_date":getdate()},fields=["unit"])
    if units:
        for unit in units:
            frappe.db.set_value("Unit", unit["unit"], "status", "Vacant")



def update_unit_status_basd_on_reservation():
    print("==========================starting============================")
    units = frappe.db.get_list("Reservations",filters={"docstatus":1,"reservation_end":["<",get_datetime()]},fields=["unit"])
    if units:
        for unit in units:
            frappe.db.set_value("Unit", unit["unit"], "status", "Vacant")
    print("==========================Finished============================")