// Copyright (c) 2021, Havenir Solutions and contributors
// For license information, please see license.txt

frappe.ui.form.on('Property Contract', {
    onload: function (frm) {
        get_brand_names(frm)
    },
    generate_payment_scheduale: function (frm) {
        frm.doc.payment = []
        frappe.call({
            method: 'add_payment',
            doc: frm.doc,
            callback: function (res) {
                frappe.show_alert('Generated Successfully')

                cur_frm.reload_doc()
            }
        })
    },
    renter: function (frm) {
        get_brand_names(frm)
    },
    refresh: function (frm) {
        get_brand_names(frm)
        frm.set_query('property_name', () => {
            return {
                filters: {
                    property_type: frm.doc.property_type
                }
            }
        })
        frm.set_query('unit', () => {
            return {
                filters: {
                    unit_type: frm.doc.unit_type,
                    property: frm.doc.property_name
                }
            }
        })
        frm.set_query('rent_item', () => {
            return {
                filters: {
                    is_sales_item: 1
                }
            }
        })
        frm.set_query('service_item', () => {
            return {
                filters: {
                    is_sales_item: 1
                }
            }
        })
        frm.clear_custom_buttons()

        if (frm.doc.docstatus == 0 && !frm.is_new()) {
            frm.add_custom_button(__("Generate Payment Scheduale"), function () {
                frm.trigger('generate_payment_scheduale');
            });
        }

        if (frm.doc.docstatus == 1 && frm.doc.status != "Hold") {
            frm.add_custom_button("Hold Contract", () => {
                frappe.call({
                    method: "muezzin.api.property_contract.change_contract_status",
                    args: {
                        doc: frm.doc,
                        status: "Hold"
                    },
                    callback: () => {
                        frm.reload_doc();
                    }
                })
            })
        }
        if (frm.doc.docstatus == 1 && frm.doc.status == "Hold") {
            frm.add_custom_button("Release Contract", () => {
                frappe.call({
                    method: "muezzin.api.property_contract.change_contract_status",
                    args: {
                        doc: frm.doc,
                        status: "Active"
                    },
                    callback: () => {
                        frm.reload_doc();
                    }
                })
            })
        }
    },

    rent_start_date: function (frm) {
        update_duration(frm)
        // if (frm.doc.docstatus == 0 && !frm.is_new()) {
        //     frm.trigger('generate_payment_scheduale');
        // }
    },
    rent_end_date: function (frm) {
        update_duration(frm)
        // if (frm.doc.docstatus == 0 && !frm.is_new()) {
        //     frm.trigger('generate_payment_scheduale');
        // }
    },

    // annual_raise_start_date: function (frm) {
    //     if (frm.doc.docstatus == 0 && !frm.is_new()) {
    //         frm.trigger('generate_payment_scheduale');
    //     }
    // },
    // service_charge_start_date: function (frm) {
    //     if (frm.doc.docstatus == 0 && !frm.is_new()) {
    //         frm.trigger('generate_payment_scheduale');
    //     }
    // },

    property_type: function (frm) {
        frm.doc.property_name = null;
        frm.refresh_field('property_name');
    },

    unit_type: function (frm) {
        frm.doc.property_name = null;
        frm.refresh_field('unit');
    },

    // duration: function(frm) {
    // 	update_payment(frm);
    // 	update_annual_raise_amount(frm);
    // },

    // type: function(frm) {
    // 	update_payment(frm);
    // },

    // payments_scheduling: function(frm) {
    // 	update_payment(frm);
    // },

    unit: function (frm) {
        update_total_unit_area(frm);
    },
    service_indoor_area: function (frm) {
        update_service_charge_per_month(frm);
    },
    service_charge_sqm: function (frm) {
        update_service_charge_per_month(frm);
    },
    outdoor_service_charge_per_month: function (frm) {

        update_total_services_rent(frm)
        update_total_rent(frm);

        // update_total_rent_month(frm);
        update_total_services_rent(frm)
        update_outdoor_service_charge_sqm(frm);
    },

    service_outdoor_area: function (frm) {
        // update_outdoor_service_charge_sqm(frm);
    },
    outdoor_service_charge_sqm: function (frm) {
        update_outdoor_service_charge_sqm(frm);
    },
    indoor_area: function (frm) {
        update_indoor_rent_month(frm);
    },

    rent_per_sqm: function (frm) {
        update_indoor_rent_month(frm);
    },

    indoor_rent_month: function (frm) {
        update_total_rent_month(frm);
    },

    // outdoor_service_charge_per_month: function(frm) {
    // 	update_total_rent_month(frm);
    // },
    outdoor_area: function (frm) {
        update_outdoor_rent_month(frm);
    },

    outdoor_rent_sqm: function (frm) {
        update_outdoor_rent_month(frm);
    },

    outdoor_rent_month: function (frm) {
        update_total_rent_month(frm);
    },

    total_rent_per_month: function (frm) {
        update_total_rent(frm);
    },

    service_charge_per_month: function (frm) {
        update_total_services_rent(frm)
        update_total_rent(frm);
    },

    total_rent: function (frm) {
        if (frm.doc.total_rent) {
            frm.set_value('annualized_rent', frm.doc.total_rent * 12);
        }
    }
});
frappe.ui.form.on('Contract Payments', {
    create_invoice: function (frm, cdt, cdn) {
        var row = locals[cdt][cdn];
        if (!row.invoice) {
            frm.call({
                method: 'create_row_invoice',
                doc: frm.doc,
                args: {rowid: cdn},
                callback: function (res) {
                    frm.reload_doc();
                }
            })
        } else {
            frappe.throw(__('Invoice Already Created..'))
        }
    }
})

const update_payment = function (frm) {
    if (!frm.doc.rent_start_date) {
        frm.doc.duration = null;
        frm.refresh_field('duration');
        frappe.msgprint('Enter Rent Start Date.')
    } else {
        if (frm.doc.type && frm.doc.duration) {
            frm.call('add_payment');
            frm.refresh_field('payments');
        } else {
            frm.doc.payments = [];
            frm.refresh_field('payments');
        }
    }
}

const update_annual_raise_amount = function (frm) {
    if (!frm.doc.rent_start_date) {
        frm.doc.duration = null;
        frm.refresh_field('duration');
        frappe.msgprint('Enter Rent Start Date.')
    } else {
        if (frm.doc.annual_raise === 'Annual Irregular' && frm.doc.duration) {
            frm.call('add_contract_annual_raise');
            frm.refresh_field('contract_annual_raise');
        } else {
            frm.doc.contract_annual_raise = [];
            frm.refresh_field('contract_annual_raise');
        }
    }
}

// const update_service_charge = function(frm) {
// 	if(frm.doc.gla && frm.doc.service_charge_sqm) {
// 		frm.set_value('service_charge_per_month', frm.doc.gla * frm.doc.service_charge_sqm)
// 	}
// }

const update_total_rent_month = function (frm) {
    if (frm.doc.indoor_rent_month >= 0 && frm.doc.outdoor_rent_month >= 0) {
        frm.set_value('total_rent_per_month', flt(frm.doc.indoor_rent_month) + flt(frm.doc.outdoor_rent_month) + flt(frm.doc.outdoor_service_charge_per_month));
    }
}

const update_total_rent = function (frm) {
    if (frm.doc.total_rent_per_month >= 0 && frm.doc.service_charge_per_month >= 0) {
        frm.set_value('total_rent', flt(frm.doc.total_rent_per_month) + flt(frm.doc.service_charge_per_month) + flt(frm.doc.outdoor_service_charge_per_month));
    }
}

const update_outdoor_rent_month = function (frm) {
    // if (frm.doc.outdoor_area && frm.doc.outdoor_rent_sqm) {
    frm.set_value('outdoor_rent_month', frm.doc.outdoor_area * frm.doc.outdoor_rent_sqm);
    // }
}

const update_indoor_rent_month = function (frm) {
    if (frm.doc.indoor_area && frm.doc.rent_per_sqm) {
        frm.set_value('indoor_rent_month', frm.doc.indoor_area * frm.doc.rent_per_sqm);
    }
}
const update_total_unit_area = function (frm) {
    // if (frm.doc.gla && frm.doc.service_charge_sqm) {
    frm.set_value('gla', frm.doc.outdoor_area + frm.doc.indoor_area);
    refresh_field('gla')
    // }
}
const update_service_charge_per_month = function (frm) {
    // if (frm.doc.gla && frm.doc.service_charge_sqm) {
    frm.set_value('service_charge_per_month', frm.doc.service_indoor_area * frm.doc.service_charge_sqm);
    // }
}
const update_total_services_rent = function (frm) {
    frm.set_value('total_services_rent', flt(frm.doc.outdoor_service_charge_per_month) + flt(frm.doc.service_charge_per_month));
}
const update_outdoor_service_charge_sqm = function (frm) {
    // if (frm.doc.outdoor_service_charge_sqm && frm.doc.service_outdoor_area) {
    // 	frm.set_value('outdoor_service_charge_sqm',0)
    // 	frm.set_value('outdoor_rent_sqm',0)
    frm.set_value('outdoor_service_charge_per_month', frm.doc.outdoor_service_charge_sqm * frm.doc.service_outdoor_area);
    // refresh_field('outdoor_rent_sqm')
    // refresh_field('outdoor_service_charge_per_month')
    // }
}
const update_duration = function (frm) {
    if (frm.doc.rent_start_date && frm.doc.rent_end_date) {
        // let duration = get_time_diff(frm.doc.rent_end_date, frm.doc.rent_start_date)
        // frm.set_value('duration', duration)

        frappe.call({
            method:'update_duration',
            doc:frm.doc,
            callback:function(r){
                if(r.message){
                    frm.set_value('duration', r.message)
                    refresh_field('duration')
                }
            }
        })
    }
}

const get_time_diff = (end, start) => {
    const end_datetime = new Date(end)
    const start_datetime = new Date(start)
    const time_diff = end_datetime - start_datetime
    const months = time_diff / 2592000000
    return Math.round(months)
}


let get_brand_names = function (frm) {
    if (frm.doc.renter) {
        frappe.call({
            method: "muezzin.property_management.doctype.property_contract.property_contract.get_brand_names",
            args: {
                renter:frm.doc.renter
            },
            callback: function (data) {
                if (data.message) {
                    let options = data.message;
                    let options_new = options.join("\n");
                    frm.set_df_property('brand_name', 'options', options_new);
                }
            }
        })
    }
}