// Copyright (c) 2021, Havenir Solutions and contributors
// For license information, please see license.txt

frappe.ui.form.on('Reservations', {
    refresh: function (frm) {
        if (frm.doc.docstatus == 1) {
            frm.add_custom_button(__("Contract"), () => {
                frm.events.create_new_contract(frm)
            }, "Create");
        }

        frm.set_query('property_name', () => {
            return {
                filters: {
                    property_type: frm.doc.property_type
                }
            }
        })

        frm.set_query('unit', () => {
            return {
                filters: {
                    status: 'Vacant'
                }
            }
        })
    },
    create_new_contract: function (frm) {
        if (frm.doc.reservation_type == 'Selling') {
            frappe.model.open_mapped_doc({
                method: "muezzin.property_management.doctype.reservations.reservations.make_unit_sale_contract",
                frm: cur_frm
            })
        } else {
            frappe.model.open_mapped_doc({
                method: "muezzin.property_management.doctype.reservations.reservations.make_property_contract",
                frm: cur_frm
            })
        }
    },
    property_type: function (frm) {
        frm.doc.property_name = null;
        frm.refresh_field('property_name');
    },
    duration: function (frm) {
        if (frm.doc.start_date) {
            let dt = frm.doc.start_date
            let hours = flt(frm.doc.duration);
            let details = SplitTime(hours)
            // alert(details)
            let slot_to_time = moment(dt).add(details).format('YYYY-MM-DD HH:mm:ss')
            frm.set_value('reservation_end', slot_to_time)
            refresh_field('reservation_end')
        }
    },
    customer_primary_address: function (frm) {
        if (frm.doc.customer_primary_address) {
            frappe.call({
                method: 'frappe.contacts.doctype.address.address.get_address_display',
                args: {
                    "address_dict": frm.doc.customer_primary_address
                },
                callback: function (r) {
                    frm.set_value("primary_address", r.message);
                }
            });
        }
        if (!frm.doc.customer_primary_address) {
            frm.set_value("primary_address", "");
        }
    },
     unit: function (frm) {
        update_total_unit_area(frm);
    },
    service_indoor_area: function (frm) {
        update_service_charge_per_month(frm);
    },
    service_charge_sqm: function (frm) {
        update_service_charge_per_month(frm);
    },
    outdoor_service_charge_per_month: function (frm) {

        update_total_services_rent(frm)
        update_total_rent(frm);

        // update_total_rent_month(frm);
        update_total_services_rent(frm)
        update_outdoor_service_charge_sqm(frm);
    },

    service_outdoor_area: function (frm) {
        // update_outdoor_service_charge_sqm(frm);
    },
    outdoor_service_charge_sqm: function (frm) {
        update_outdoor_service_charge_sqm(frm);
    },
    indoor_area: function (frm) {
        update_indoor_rent_month(frm);
    },

    rent_per_sqm: function (frm) {
        update_indoor_rent_month(frm);
    },

    indoor_rent_month: function (frm) {
        update_total_rent_month(frm);
    },

    // outdoor_service_charge_per_month: function(frm) {
    // 	update_total_rent_month(frm);
    // },
    outdoor_area: function (frm) {
        update_outdoor_rent_month(frm);
    },

    outdoor_rent_sqm: function (frm) {
        update_outdoor_rent_month(frm);
    },

    outdoor_rent_month: function (frm) {
        update_total_rent_month(frm);
    },

    total_rent_per_month: function (frm) {
        update_total_rent(frm);
    },

    service_charge_per_month: function (frm) {
        update_total_services_rent(frm)
        update_total_rent(frm);
    },

    total_rent: function (frm) {
        if (frm.doc.total_rent) {
            frm.set_value('annualized_rent', frm.doc.total_rent * 12);
        }
    }
});


let SplitTime = function (numberOfHours) {
    var Days = Math.floor(numberOfHours / 24);
    var Remainder = numberOfHours % 24;
    var Hours = Math.floor(Remainder);
    var Minutes = Math.floor(60 * (Remainder - Hours));
    return ({"days": Days, "hours": Hours, "minutes": Minutes})
}

const update_indoor_rent_month = function (frm) {
    if (frm.doc.indoor_area && frm.doc.rent_per_sqm) {
        frm.set_value('indoor_rent_month', frm.doc.indoor_area * frm.doc.rent_per_sqm);
    }
}

const update_service_charge_per_month = function (frm) {
    // if (frm.doc.gla && frm.doc.service_charge_sqm) {
    frm.set_value('service_charge_per_month', frm.doc.service_indoor_area * frm.doc.service_charge_sqm);
    // }
}

const update_outdoor_service_charge_sqm = function (frm) {
    frm.set_value('outdoor_service_charge_per_month', frm.doc.outdoor_service_charge_sqm * frm.doc.service_outdoor_area);
}

const update_outdoor_rent_month = function (frm) {
    frm.set_value('outdoor_rent_month', frm.doc.outdoor_area * frm.doc.outdoor_rent_sqm);
}

const update_total_rent = function (frm) {
    if (frm.doc.total_rent_per_month >= 0 && frm.doc.service_charge_per_month >= 0) {
        frm.set_value('total_rent', flt(frm.doc.total_rent_per_month) + flt(frm.doc.service_charge_per_month) + flt(frm.doc.outdoor_service_charge_per_month));
    }
}

const update_total_services_rent = function (frm) {
    frm.set_value('total_services_rent', flt(frm.doc.outdoor_service_charge_per_month) + flt(frm.doc.service_charge_per_month));
}

const update_total_unit_area = function (frm) {
    // if (frm.doc.gla && frm.doc.service_charge_sqm) {
    frm.set_value('gla', frm.doc.outdoor_area + frm.doc.indoor_area);
    refresh_field('gla')
    // }
}

const update_total_rent_month = function (frm) {
    if (frm.doc.indoor_rent_month >= 0 && frm.doc.outdoor_rent_month >= 0) {
        frm.set_value('total_rent_per_month', flt(frm.doc.indoor_rent_month) + flt(frm.doc.outdoor_rent_month) + flt(frm.doc.outdoor_service_charge_per_month));
    }
}
