// Copyright (c) 2021, Havenir Solutions and contributors
// For license information, please see license.txt

frappe.ui.form.on('Unit', {
	onload(frm) {
		frm.set_query("sub_activity", () => {
			return {
				filters: {
					parent_activity: frm.doc.unit_activity
				}
			}
		})
	},
	refresh: function (frm) {
		if (!frm.is_dirty()) {
			create_custom_buttons(frm);
		}
	},

	unit_area: function (frm) {
		update_gla(frm);
	},

	outdoor_area: function (frm) {
		update_gla(frm);
	},

	gla: function (frm) {
		if (frm.doc.gla < 0) {
			frm.doc.unit_price = null;
			frm.doc.gla = null;
			frm.refresh_field('gla');
			frappe.msgprint('Unit Area must be greater than zero.')
		} else if (frm.doc.gla && frm.doc.price_psm) {
			frm.doc.unit_price = frm.doc.gla * frm.doc.price_psm;
		}
		frm.refresh_field('unit_price');
	},

	price_psm: function (frm) {
		if (frm.doc.price_psm < 0) {
			frm.doc.price_psm = null;
			frm.doc.unit_price = null;
			frm.refresh_field('price_psm');
			frappe.msgprint('Price must be greater than zero.')
		} else if (frm.doc.gla && frm.doc.price_psm) {
			frm.doc.unit_price = frm.doc.gla * frm.doc.price_psm;
		}
		frm.refresh_field('unit_price');
	}
});

const create_custom_buttons = function(frm) {
	frm.add_custom_button(__('Journal Entry'), () => {
		let doc = frm.doc
		frappe.run_serially([
			() => frappe.new_doc('Journal Entry'),
			() => {
				cur_frm.doc.cost_center = doc.cost_center;
				cur_frm.doc.property = doc.property;
				cur_frm.doc.unit = doc.name;
				cur_frm.refresh();
			}
		]);
	}, __('Create'));

	cur_frm.add_custom_button(__('Sales Invoice'), () => make_sales_invoice(), __('Create'));

	// frm.add_custom_button(__('Sales Invoice'), () => {
	// 	let doc = frm.doc
	// 	frappe.run_serially([
	// 		() => frappe.new_doc('Sales Invoice'),
	// 		() => {
	// 			cur_frm.doc.cost_center = doc.cost_center;
	// 			cur_frm.doc.property = doc.property;
	// 			cur_frm.doc.unit = doc.name;
	// 			let customer = doc.property_owner_customer;
	// 			frappe.model.set_value(cur_frm.doc.doctype, cur_frm.doc.name, "customer", customer);
	// 			cur_frm.refresh();
	// 		}
	// 	]);
	// }, __('Create'));

	frm.add_custom_button(__('Payment Entry'), () => {
		let doc = frm.doc
		frappe.run_serially([
			() => frappe.new_doc('Payment Entry'),
			() => {
				cur_frm.doc.cost_center = doc.cost_center;
				cur_frm.doc.property = doc.property;
				cur_frm.doc.unit = doc.name;
				
				if (doc.property_owner_customer) {
					cur_frm.doc.party_type = 'Customer';
					let party = doc.property_owner_customer;
					frappe.model.set_value("Payment Entry", cur_frm.doc.name, "party", party)
				}

				if (doc.property_owner_supplier) {
					cur_frm.doc.party_type = 'Supplier';
					let party = doc.property_owner_supplier;
					frappe.model.set_value("Payment Entry", cur_frm.doc.name, "party", party)
				}
				cur_frm.refresh();
			}
		]);
		
	}, __('Create'));

	frm.add_custom_button(__('Purchase Invoice'), () => {
		let doc = frm.doc
		frappe.run_serially([
			() => frappe.new_doc('Purchase Invoice'),
			() => {
				cur_frm.doc.cost_center = doc.cost_center;
				cur_frm.doc.property = doc.property;
				cur_frm.doc.unit = doc.name;
				let supplier = doc.property_owner_supplier;
				frappe.model.set_value(cur_frm.doc.doctype, cur_frm.doc.name, "supplier", supplier);
				cur_frm.refresh();
			}
		]);
	}, __('Create'));

	frm.add_custom_button(__('Contract'), () => {
		let doc = frm.doc
		frappe.run_serially([
			() => frappe.new_doc('Property Contract'),
			() => {
				cur_frm.doc.property_name = doc.property;
				cur_frm.doc.unit_name = doc.name;
				cur_frm.refresh();
			}
		]);
	}, __('Create'));
}

const update_gla = function(frm) {
	if (frm.doc.unit_area>=0 && frm.doc.outdoor_area>=0) {
		frm.set_value('gla', frm.doc.unit_area+frm.doc.outdoor_area);
	}
}


var make_sales_invoice = function () {
		frappe.model.open_mapped_doc({
			method: "muezzin.property_management.doctype.unit.unit.make_sales_invoice",
			frm: cur_frm
		});
	}