frappe.pages["property-dashboard"].on_page_load = function (wrapper) {
  frappe.property_dashboard = new PropertyDashboard(wrapper);
};

class PropertyDashboard {
  constructor(wrapper) {
    this.page = wrapper.page;

    frappe.run_serially([
      () => this.make_page(wrapper),
      () => this.make_action_bar(wrapper),
      () => this.make_charts(wrapper),
    ]);
  }

  make_page(wrapper) {
    this.page = frappe.ui.make_app_page({
      parent: wrapper,
      title: "Property Dashboard",
      single_column: true,
    });
  }

  make_action_bar(wrapper) {
    this.page.set_secondary_action(
      "Refresh",
      () => this.make_charts(wrapper),
      "octicon octicon-sync"
    );
  }

  make_charts(wrapper) {
    const layout = $(wrapper).find(".layout-main-section");

    // server call to get data
    frappe.call({
      method: "muezzin.api.property_dashboard.get_tiles_data",
      freeze: true,
      callback: (r) => {
        const data = r.message;
        const context = { tiles: data };

        const tiles = frappe.render_template("dashboard-tiles", context);
        layout.html("");
        layout.html(tiles);
      },
      error: (r) => {
        const error = "<p class='text-danger text-center'>System error</p>";
        layout.append(error);
        console.error(r);
      },
    });
  }
}
