# Copyright (c) 2013, Havenir Solutions and contributors
# For license information, please see license.txt

from __future__ import unicode_literals
import frappe
from frappe import _
from dateutil import relativedelta
import datetime
from dateutil.rrule import rrule, MONTHLY
from frappe.utils import date_diff, flt

def execute(filters=None):
    columns = [
        {
            "fieldname": "location",
            "label": _("Location"),
            "fieldtype": "Data"
        },
        {
            "fieldname": "contract",
            "label": _("Contract"),
            "fieldtype": "Link",
            "options": "Property Contract"
        },
        {
            "fieldname": "insurance_policy",
            "label": _("Insurance policy"),
            "fieldtype": "Data"
        },
        {
            "fieldname": "revenue_sharing",
            "label": _("Revenue sharing"),
            "fieldtype": "Data"
        },
        {
            "fieldname": "renter",
            "label": _("Legal Tenant Name"),
            "fieldtype": "Data"
        },
        {
            "fieldname": "brand_name",
            "label": _("Tenant Brand Name"),
            "fieldtype": "Data"
        },
        # {
        #     "fieldname": "owner",
        #     "label": _("Owner"),
        #     "fieldtype": "Link",
        #     "options": "Customer"
        # },
        {
            "fieldname": "unit",
            "label": _("Unit No."),
            "fieldtype": "Link",
            "options": "Unit"
        },
        # {
        #     "fieldname": "unit_status",
        #     "label": _("Unit Status"),
        #     "fieldtype": "Data"
        # },
        # {
        #     "fieldname": "unit_owner",
        #     "label": _("Tenant Name"),
        #     "fieldtype": "Link",
        #     "options": "Customer"
        # },
        {
            "fieldname": "property",
            "label": _("Bldg. No"),
            "fieldtype": "Link",
            "options": "Property"
        },
        {
            "fieldname": "opening_date",
            "label": _("Opening Date"),
            "fieldtype": "Date"
        },
        {
            "fieldname": "role",
            "label": _("Floor"),
            "fieldtype": "Data"
        },
        {
            "fieldname": "unit_type",
            "label": _("Unit Type"),
            "fieldtype": "Data"
        },
        # {
        #     "fieldname": "unit_activity",
        #     "label": _("Unit Activity"),
        #     "fieldtype": "Data"
        # },
        # {
        #     "fieldname": "unit_subactivity",
        #     "label": _("Unit Subactivity"),
        #     "fieldtype": "Data"
        # },
        {
            "fieldname": "indoor_area",
            "label": _("Indoor Area(m2)"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "outdoor_area",
            "label": _("Outdoor Area"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "gla",
            "label": _("GLA"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "rent_per_sqm",
            "label": _("Rent Per Sqm"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "indoor_rent",
            "label": _("Indoor Rent LE/month"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "outdoor_rent_per_sqm",
            "label": _("Outdoor Rent SQM"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "outdoor_rent",
            "label": _("Outdoor Rent LE / month"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "total_rent",
            "label": _("TOTAL RENT/Month"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "service_charge_per_sqm",
            "label": _("Service Charge SQM"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "service_charge",
            "label": _("Service Charge Month"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "outdoor_service_charge_per_month",
            "label": _("OutDoor Service Per Month"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "total_services_rent",
            "label": _("Total Services Charges"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "total_rent_plus_service_charge",
            "label": _("TOTAL RENT/Month + SC"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "annualized_rent",
            "label": _("Annualized Rent"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "annual_raise_amount",
            "label": _("Annual Raise"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "annual_raise_date",
            "label": _("Date of Increase"),
            "fieldtype": "Date"
        },
        {
            "fieldname": "opening_phase",
            "label": _("Opening Phase"),
            "fieldtype": "Float"
        },
        {
            "fieldname": "lease_sent",
            "label": _("Lease Sent"),
            "fieldtype": "Check"
        },
        {
            "fieldname": "lease_signed",
            "label": _("Lease Signed"),
            "fieldtype": "Check"
        },
        {
            "fieldname": "rf_deposit",
            "label": _("RF Deposit"),
            "fieldtype": "Float",
            "default": 0
        },
        {
            "fieldname": "rent_sold",
            "label": _("Rent / Sold"),
            "fieldtype": "Data"
        },
        {
            "fieldname": "marketing_fees",
            "label": _("Marketing Fees"),
            "fieldtype": "Data"
        },
        {
            "fieldname": "insurance_payment",
            "label": _("Insurance Payment"),
            "fieldtype": "Data"
        },
        {
            "fieldname": "lease_start",
            "label": _("Lease Start"),
            "fieldtype": "Date"
        },
        {
            "fieldname": "lease_term",
            "label": _("Lease Term(in yrs)"),
            "fieldtype": "Float",
            "precision": "2"
        },
        {
            "fieldname": "lease_end",
            "label": _("Lease End"),
            "fieldtype": "Date"
        },
        {
            "fieldname": "months_left",
            "label": _("Month Left"),
            "fieldtype": "Data"
        },
        {
            "fieldname": "years_left",
            "label": _("Year Left"),
            "fieldtype": "Data"
        }
    ]

    start_date = datetime.datetime.strptime(str(filters.from_date), "%Y-%m-%d")
    end_date = datetime.datetime.strptime(str(filters.to_date), "%Y-%m-%d")

    for row in rrule(MONTHLY, dtstart=start_date, until=end_date):
        month = datetime.datetime.strptime(str(row.month), "%m")
        month_name = month.strftime("%b")

        month = '{:02d}'.format(row.month)
        day = '{:02d}'.format(row.day)

        label = f'{day}-{month_name}-{row.year}'
        fieldname = f'{day}-{month}-{row.year}'
        columns.append(
            {
                "fieldname": fieldname,
                "label": _(label),
                "fieldtype": "Data"
            }
        )
    data = get_data(filters)
    return columns, data


def get_data(filters):
    data = []
    q_filter = {}
    if filters.docstatus == "Submitted":
        q_filter = {"docstatus": 1}
    elif filters.docstatus == "Draft":
        q_filter = {"docstatus": 0}
    else:
        q_filter = {"docstatus": ["!=", "2"]}

    result = frappe.get_list('Property Contract', q_filter, ['name'])
    for row in result:
        contract = frappe.get_doc('Property Contract', row)
        if not contract.unit:
            continue
        unit = frappe.get_doc('Unit', contract.unit)
        prop = frappe.get_doc('Property', contract.property_name)

        # Calculate Month Left
        date1 = datetime.datetime.strptime(str(contract.rent_end_date), '%Y-%m-%d')
        date2 = datetime.datetime.today()
        date_diffrance = date_diff(date2, date1)

        r = relativedelta.relativedelta(date1, date2)
        # frappe.msgprint(str(r))
        months_left = abs(r.months)
        years_left = abs(r.years)

        if date_diffrance > 0:
            months_left = 0
            years_left = 0

        col_data = {
            'location': prop.property_location,
            'brand_name': contract.brand_name,
            'insurance_policy': contract.insurance_policy,
            'revenue_sharing': contract.revenue_sharing,
            'contract': contract.name,
            'renter': contract.renter,
            # 'owner': prop.property_owner,
            'unit': unit.name,
            # 'unit_status': unit.status,
            # 'unit_owner': unit.property_owner,
            'property': unit.property,
            'role': unit.role_floor_number,
            'unit_type': unit.unit_type,
            # 'unit_activity': unit.unit_activity,
            # 'unit_subactivity': unit.sub_activity,
            'indoor_area': unit.unit_area,
            'outdoor_area': unit.outdoor_area,
            'gla': unit.gla,
            'rent_per_sqm': contract.rent_per_sqm,
            'indoor_rent': contract.indoor_rent_month,
            'outdoor_rent_per_sqm': contract.outdoor_rent_sqm,
            'outdoor_rent': contract.outdoor_rent_month,
            'total_rent': contract.total_rent_per_month,
            'service_charge_per_sqm': contract.service_charge_sqm,
            'service_charge': contract.service_charge_per_month,
            'outdoor_service_charge_per_month': contract.outdoor_service_charge_per_month,
            'total_services_rent': contract.total_services_rent,
            'total_rent_plus_service_charge': contract.total_rent,
            'annualized_rent': flt(contract.total_rent_per_month) * 12,  # contract.annualized_rent,
            'annual_raise_amount': contract.annual_raise_amount,
            'annual_raise_date': contract.annual_raise_start_date,
            'opening_date': contract.opening_date,
            'opening_phase': unit.opening_phase,
            'lease_sent': contract.lease_sent,
            'lease_signed': contract.lease_signed,
            'rf_deposit': 0,
            'rent_sold': unit.status,
            'marketing_fees': contract.marketing_fees,
            'insurance_payment': contract.insurance_payment,
            'lease_start': contract.rent_start_date,
            'lease_term': contract.duration / 12,
            'lease_end': contract.rent_end_date,
            'months_left': months_left,
            'years_left': years_left
        }

        # Fetch monthly payments from payment table in property contract
        # Add value in respective date column

        contract_months = {}

        for row in contract.payments:
            date = row.date.strftime("%d-%m-%Y")
            if date in contract_months:
                contract_months[date] += row.amount
            else:
                contract_months[date] = row.amount  # Make dict of all payments of current contract
            # print(str(date)+"============="+str(contract_months[date]))

        start_date = datetime.datetime.strptime(str(filters.from_date), "%Y-%m-%d")
        end_date = datetime.datetime.strptime(str(filters.to_date), "%Y-%m-%d")

        date_diffrance = date_diff(start_date, end_date)
        if date_diffrance < 0:
            end_date += relativedelta.relativedelta(days=abs(date_diffrance))

        for row in rrule(MONTHLY, dtstart=start_date, until=end_date):
            month = '{:02d}'.format(row.month)
            day = '{:02d}'.format(row.day)

            fieldname = f'{day}-{month}-{row.year}'

            if fieldname in contract_months and contract_months[fieldname]:
                # frappe.msgprint(str(contract_months[fieldname]))
                col_data[fieldname] = contract_months[fieldname]

        # frappe.msgprint(str(col_data))
        ## NOTE
        # check if this label exists in dict of payments made above
        # if exists then payment for this contract is there so append it in columns
        # all that's left is correct syntax to append in columns where date is same as column label
        # And calculation of yearly payment if required

        data.append(col_data)
    return data
