# Copyright (c) 2013, Havenir Solutions and contributors
# For license information, please see license.txt

from __future__ import unicode_literals
import frappe

def execute(filters=None):
	columns = get_columns(filters)
	data = build_data(filters)
	return columns, data

def get_columns(filters):
	columns = [
		{	
			"label": "Sr",
			"fieldname": "serial_number",
			"fieldtype": "Int",
			"width": 50
		},
		{	
			"label": "Name",
			"fieldname": "unit",
			"fieldtype": "Link",
			"options": "Unit",
			"width": 100
		},
		{	
			"label": "Docstatus",
			"fieldname": "docstatus",
			"fieldtype": "Int",
			"width": 100
		},
		{	
			"label": "Unit Number",
			"fieldname": "unit_number",
			"fieldtype": "Data",
			"width": 100
		},
		{	
			"label": "Role (Floor Number)",
			"fieldname": "role_floor_number",
			"fieldtype": "Data",
			"width": 150
		},
		{	
			"label": "Contract Year",
			"fieldname": "contract_year",
			"fieldtype": "Data",
			"width": 100
		},
		{	
			"label": "Unit Type",
			"fieldname": "unit_type",
			"fieldtype": "Link",
			"options": "Unit Type",
			"width": 100
		},
		{	
			"label": "Property Owner (Customer)",
			"fieldname": "property_owner",
			"fieldtype": "Data",
			"width": 200
		},
		{	
			"label": "Unit Area (per Square Meter)",
			"fieldname": "unit_area",
			"fieldtype": "Int",
			"width": 200
		},
		{	
			"label": "Price (per Square Meter)",
			"fieldname": "price_psm",
			"fieldtype": "Data",
			"width": 200
		},
		{	
			"label": "Unit Price",
			"fieldname": "unit_price",
			"fieldtype": "Float",
			"width": 100
		},
		{	
			"label": "Unit Total Cost",
			"fieldname": "unit_valuation_rate",
			"fieldtype": "Float",
			"width": 100
		},
		{	
			"label": "Total Invoiced",
			"fieldname": "total_invoiced",
			"fieldtype": "Float",
			"width": 100
		},
		{	
			"label": "Total Paid",
			"fieldname": "total_paid",
			"fieldtype": "Float",
			"width": 100
		},
		{	
			"label": "Unit Profit",
			"fieldname":"unit_profit",
			"fieldtype": "Float",
			"width": 100
		},
		{	
			"label": "Total Outstanding",
			"fieldname": "total_outstanding",
			"fieldtype": "Float",
			"width": 150
		},
	]
	
	return columns
	
def build_data(filters):
	data = []
	
	sub_filters = {}
	fields = [
		"name",
		"docstatus",
		"unit_number",
		"unit_type",
		"role_floor_number",
		"property_owner_customer",
		"unit_area",
		"price_psm",
		"unit_price",
		"unit_valuation_rate",
	]
	units = frappe.get_list("Unit", sub_filters, fields)
	
	i = 1
	for row in units:
		sub_filters = {
			"unit": row.name,
			"docstatus": 1
		}

		fields = [
			"grand_total",
			"outstanding_amount",
		]
		
		invoiced = 0
		outstanding = 0
		paid = 0
		
		invoices = frappe.get_list("Sales Invoice", sub_filters, fields)
		for invoice in invoices:
			invoiced += invoice.grand_total
			outstanding += invoice.outstanding_amount
		
		paid = invoiced - outstanding
		
		unit_profit = row.unit_valuation_rate-invoiced
		
		record = {
			"serial_number": i,
			"unit": row.name,
			"docstatus": row.docstatus,
			"unit_number": row.unit_number,
			"role_floor_number": row.role_floor_number,
			"unit_type": row.unit_type,
			"property_owner": row.property_owner_customer,
			"unit_area": row.unit_area,
			"price_psm": row.price_psm,
			"unit_price": row.unit_price,
			'unit_valuation_rate':row.unit_valuation_rate,
			"total_invoiced": invoiced,
			"total_paid": paid,
			"unit_profit":unit_profit,
			"total_outstanding": outstanding,
		}
		
		i = i + 1
		
		data.append(record)
	return data